import csv
import os
import shutil
import copy
import sys, getopt

FIFA = "FIFA 21"
FA_TEAMID = 111592
SOCCER_AID_TEAMID = 114815
MAX_PLAYERS = 26000
MAX_TEAMPLAYERLINKS = 27000
MAX_EDITEDPLAYERNAMES = 1530
DEFAULT_ICONS_VERSION = "ALL"
ALLOWED_ICONS_VERSION = [
    'ALL', 'BASE', 'MIDDLE',
    'PRIME', 'OTHER'
]

def main(argv):
    icons_version = None
    try:
        opts, args = getopt.getopt(argv, "o:", ["origin="])
    except getopt.GetoptError:
        print("add_icons.py -o <origin>")
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-o", "--origin"):
            arg = arg.upper()
            if arg in ALLOWED_ICONS_VERSION:
                icons_version = arg
            else:
                print("Allowed origins, one of: {}".format(ALLOWED_ICONS_VERSION))
                sys.exit(2)

    if not icons_version:
        icons_version = DEFAULT_ICONS_VERSION

    print("Work in progress... please wait.")

    # Script working directory
    root_path = os.path.join(os.getcwd(), FIFA)

    icons_file_path = os.path.join(root_path, 'FIFA 21_ICONS.csv')
    if not os.path.exists(icons_file_path):
        print("{} file does not exist. Run the create_FIFA_ICONS_CSV.py script first to create it, "
              "then try again.".format(icons_file_path))

    # Path to folder with database tables extracted from cards_ng_db.db
    cards_ng_db_path = os.path.join(root_path, "cards_ng_db")

    # Path to players table exported from cards_ng_db.db
    players_table = os.path.join(cards_ng_db_path, "players.txt")

    # Path to folder with database tables extracted from cards_ng_db.db
    squadfile_path = os.path.join(root_path, "squadfile")
    squadfile_players = os.path.join(squadfile_path, 'players.txt')
    squadfile_teamplayerlinks = os.path.join(squadfile_path, 'teamplayerlinks.txt')

    # Result players file
    result_players = os.path.join(os.getcwd(), 'result_players.txt')

    # Result teamplayerlinks file
    result_teamplayerlinks = os.path.join(os.getcwd(), 'result_teamplayerlinks.txt')

    # Result editedplayernames file
    result_editedplayernames = os.path.join(os.getcwd(), 'result_editedplayernames.txt')

    icons_to_add = dict()

    with open(icons_file_path, 'r+', encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter=',')
        for row in reader:
            if icons_version == 'ALL' or icons_version == row['origin']:
                icons_to_add[row['playerid']] = row

    new_squadfile_players = list()
    empty_row = None
    with open(squadfile_players, 'r+', encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        for row in reader:
            if row['playerid'] == '0':
                if not empty_row:
                    empty_row = row
                continue
            # Don't duplicate players
            if row['playerid'] not in icons_to_add.keys():
                new_squadfile_players.append(row)

    fieldnames = list()
    with open(players_table, encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        fieldnames = reader.fieldnames
        for row in reader:
            if row['playerid'] not in icons_to_add.keys():
                continue
            """
            KlaiKeT custom career icons
            """
            for playerid, modified_values in icons_to_add.items():
                if playerid == row['playerid']:
                    for data in row:
                        if data in modified_values:
                            row[data] = modified_values[data]
            """
            KlaiKeT custom career icons
            """
            new_squadfile_players.append(row)
    new_squadfile_players = sorted(new_squadfile_players, reverse=False, key=lambda row: int(row['playerid']))

    empty_rows_to_create = MAX_PLAYERS - len(new_squadfile_players)
    assert empty_rows_to_create >= 0, "Too many players"

    with open(result_players, 'w+', encoding='utf-16', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames, delimiter='	')
        writer.writeheader()
        for row in new_squadfile_players:
            writer.writerow(row)

        for _ in range(empty_rows_to_create):
            writer.writerow(empty_row)

    empty_row = None
    last_row = None
    new_teamplayerlinks = list()

    teamplayerlinks_to_create = copy.copy(icons_to_add)
    with open(squadfile_teamplayerlinks, 'r+', encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        fieldnames = reader.fieldnames
        for row in reader:
            if row['playerid'] == '0':
                if not empty_row:
                    empty_row = row
                continue

            if row['playerid'] in teamplayerlinks_to_create.keys():
                if int(row['teamid']) == SOCCER_AID_TEAMID:
                    row['teamid'] = FA_TEAMID
                    del teamplayerlinks_to_create[row['playerid']]
            last_row = row
            new_teamplayerlinks.append(row)

    last_pk = int(last_row['artificialkey'])
    for playerid, _ in teamplayerlinks_to_create.items():
        last_pk += 1
        row = copy.copy(last_row)
        row.update({
            'artificialkey': last_pk,
            'playerid': playerid,
            'jerseynumber': 99,
            'position': 28,
            'teamid': FA_TEAMID
        })
        new_teamplayerlinks.append(row)

    new_teamplayerlinks = sorted(new_teamplayerlinks, reverse=False, key=lambda row: int(row['artificialkey']))

    empty_rows_to_create = MAX_TEAMPLAYERLINKS - len(new_teamplayerlinks)
    assert empty_rows_to_create >= 0, "Too many teamplayerlinks"
    with open(result_teamplayerlinks, 'w+', encoding='utf-16', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames, delimiter='	')
        writer.writeheader()
        for row in new_teamplayerlinks:
            writer.writerow(row)

        for _ in range(empty_rows_to_create):
            writer.writerow(empty_row)

    fieldnames = [
        'firstname',
        'commonname',
        'playerjerseyname',
        'surname',
        'playerid',
    ]
    empty_row = {
        'commonname': '',
        'firstname': '',
        'playerid': 0,
        'playerjerseyname': '',
        'surname': ''
    }
    empty_rows_to_create = MAX_EDITEDPLAYERNAMES - len(icons_to_add)
    assert empty_rows_to_create >= 0, "Too many edited player names"
    with open(result_editedplayernames, 'w+', encoding='utf-16', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames, delimiter='	')
        writer.writeheader()
        for k, v in icons_to_add.items():
            writer.writerow({
                'commonname': v['commonname'],
                'firstname': v['firstname'],
                'playerid': v['playerid'],
                'playerjerseyname': v['playerjerseyname'],
                'surname': v['lastname'],
            })

        for _ in range(empty_rows_to_create):
            writer.writerow(empty_row)

    print("Job done")


if __name__ == '__main__':
    main(sys.argv[1:])
