import csv
import os
import copy
import time
import hashlib
import random

FIFA = "FIFA 21"
ICONS_TEAMID = 112658
#  Stats ids to be modified
PLAYER_STATS = ["volleys", "vision", "strength", "standingtackle", "sprintspeed", "stamina","slidingtackle", "shotpower", "shortpassing", "positioning", "penalties", "marking","longshots", "longpassing", "jumping","interceptions", "headingaccuracy", "freekickaccuracy", "finishing", "dribbling", "curve", "crossing", "balance","composure", "agility","aggression", "acceleration", "gkreflexes", "gkpositioning", "gkkicking", "gkhandling", "gkdiving"]
BIRTHDATES = [154342, 153977, 153611, 153246, 152881, 152516] #  Pool of birthdates by hand. Those correspond to 2005, 2004, 2003, 2002, 2001, and 2000.

def main():
    # Script working directory
    root_path = os.path.join(os.getcwd(), FIFA)

    # Path to folder with database tables extracted from cards_ng_db.db
    cards_ng_db_path = os.path.join(root_path, "cards_ng_db")

    # Path to teamplayerlinks table exported from cards_ng_db.db
    tplinks_table = os.path.join(cards_ng_db_path, "teamplayerlinks.txt")

    # Path to players table exported from cards_ng_db.db
    players_table = os.path.join(cards_ng_db_path, "players.txt")

    # Path to playernames table exported from cards_ng_db.db
    playernames_table = os.path.join(cards_ng_db_path, "playernames.txt")

    # Output file
    output_file = os.path.join(root_path, "{}_ICONS.csv".format(FIFA))
    print(output_file)

    # Get the icons from database cards_ng_db.db
    # Table teamplayerlinks
    # All icons are in team with teamid == 112658
    # That's probably the easiest way to get playerid of all icons
    icons = dict()
    with open(tplinks_table, encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        for row in reader:
            if int(row['teamid']) != ICONS_TEAMID:
                continue

            icons[row['playerid']] = dict()

    # Just print how many icons has been found
    # Expect to be 303 in FIFA 21
    icons_found = len(icons)
    print("Icons found: {}".format(icons_found))

    # Now let's find our icons in players table
    with open(players_table, encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        player_count = 0
        for row in reader:
            if row['playerid'] not in icons:
                continue

            player_count += 1
            icons[row['playerid']].update(row)

    # If AssertionError we don't are missing some icons in players table
    print("player_count: {}".format(player_count))
    assert icons_found == player_count, 'Whoops...'

    # group the same players
    icons_grouped = dict()
    for k, v in icons.items():
        tohash = '{}-{}-{}'.format(
            v['firstnameid'], v['lastnameid'], v['commonnameid'], v['birthdate']
        )
        md5 = hashlib.md5(tohash.encode()).hexdigest()
        if md5 not in icons_grouped:
            icons_grouped[md5] = dict()
        icons_grouped[md5][k] = v

    icons_grouped_copy = copy.deepcopy(icons_grouped)

    # Just Validate
    # for k, v in icons_grouped_copy.items():
    #     # Each icon should have 3 versions (base, middle, prime)
    #     assert len(v) == 3, 'Whoops... {}'.format(len(v))

    # Next step is to find names of our icons
    with open(playernames_table, encoding='utf-16') as csvfile:
        reader = csv.DictReader(csvfile, delimiter='	')
        name_cols = [
            'firstnameid',
            'lastnameid',
            'commonnameid',
            'playerjerseynameid'
        ]

        # TODO Optimalization?
        for row in reader:
            for md5, player in icons_grouped_copy.items():
                for playerid, cols in player.items():
                    for col in name_cols:
                        if cols[col] == row['nameid']:
                            icons_grouped[md5][playerid].update({
                                col.replace('id', ''): row['name']
                            })

    # Create the result file
    with open(output_file, 'w+', encoding='utf-16', newline='') as csvfile:
        versions = [
            'PRIME',
            'MIDDLE',
            'BASE'
        ]

        fieldnames = [
            'playerid',
            'origin',
            'overallrating',
            'firstname',
            'lastname',
            'playerjerseyname',
            'commonname',
            'firstnameid',
            'lastnameid',
            'commonnameid',
            'playerjerseynameid',
            'birthdate', #  Added by KlaiKeT
            'potential' #  Added by KlaiKeT
        ]
        fieldnames.extend(PLAYER_STATS)
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        for md5, player in icons_grouped.items():
            idx = 0
            for playerid, cols in sorted(player.items(), reverse=True, key=lambda x: int(x[1]['overallrating'])):
                row = dict()
                try:
                    cols['origin'] = versions[idx]
                except IndexError:
                    cols['origin'] = "OTHER"    # WTF Hierro got 4 versions?
                for field in fieldnames:
                    row[field] = cols[field]
                    """
                    Change statistics and birthdate
                    """
                    if field in PLAYER_STATS:
                        row[field] = str(int(int(cols[field]) * 0.77)) # Icons get 77% of their original stats.
                    if field == "overallrating":
                        row[field] = str(int(int(cols[field]) * 0.83)) # Overall 83% to account for the random S#!T Fifa does to calculate this.
                    if field == "birthdate":
                        row[field] = str(random.choice(BIRTHDATES))
                writer.writerow(row)
                idx += 1


if __name__ == '__main__':
    start_time = time.time()
    main()
    print("--- %s seconds ---" % (time.time() - start_time))



